#include <string.h>
#include <glib.h>
#include "../include/string.h"
#include "obj.h"


obj_struct *ObjNew(void);
obj_struct *ObjNewFromPath(const gchar *path);
obj_struct *ObjCopy(const obj_struct *obj);
void ObjDelete(obj_struct *obj);

GList *ObjDDEBufferParse(const guint8 *buf, gint buf_len);
guint8 *ObjDDEBufferAppend(
	guint8 *buf, gint *buf_len,
	const obj_struct *obj  
);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Creates a new Object.
 */
obj_struct *ObjNew(void)
{
	return(
	    OBJ(g_malloc0(sizeof(obj_struct)))
	);

}

/*
 *	Creates a new Object from the specified path.
 */
obj_struct *ObjNewFromPath(const gchar *path)
{
	obj_struct *obj = ObjNew();
	if(obj == NULL)
	    return(NULL);

	if(path != NULL)
	{
	    obj->name = STRDUP(g_basename(path));
	    obj->full_path = STRDUP(path);
	}

	return(obj);
}

/*
 *	Coppies the Object.
 */
obj_struct *ObjCopy(const obj_struct *obj)
{
	const obj_struct *src = obj;
	obj_struct *tar = ObjNew();
	if(tar == NULL)
	    return(NULL);

	tar->name = STRDUP(src->name);
	tar->full_path = STRDUP(src->full_path);

	return(tar);
}

/*
 *	Deletes the Object.
 */
void ObjDelete(obj_struct *obj)
{
	if(obj == NULL)
	    return;

	g_free(obj->name);
	g_free(obj->full_path);
	g_free(obj);
}


/*
 *	Parses the DDE buffer into a list of Objects.
 */
GList *ObjDDEBufferParse(const guint8 *buf, gint buf_len)
{
	gint len;
	gchar *url;
	const gchar *s;
	const guint8 *buf_end = buf + buf_len;
	GList *glist = NULL;
	obj_struct *obj;

	if(buf == NULL)
	    return(glist);

	/* Iterate through buffer */
	while(buf < buf_end)
	{
	    /* Create a new Object and append it to the list */
	    obj = ObjNew();
	    if(obj == NULL)
		break;

	    glist = g_list_append(glist, obj);

	    /* Seek s to end of url */
	    s = (const gchar *)buf;
	    while(s < (const gchar *)buf_end)
	    {
		if(*s == '\0')
		    break;
		s++;
	    }

	    /* Get length of url */
	    len = (gint)(s - (const gchar *)buf);

	    /* Allocate url */
	    url = (gchar *)g_malloc((len + 1) * sizeof(gchar));
	    if(url == NULL)
		break;
	    memcpy(url, buf, len * sizeof(gchar));
	    url[len] = '\0';

	    /* Get full path from url, seek past "file://" prefix */
	    if(strpfx(url, "file://"))
	    {
		gchar	*s2 = url + STRLEN("file://"),
			*s3 = strchr(s2, '/');
		if(s3 != NULL)
		    obj->full_path = STRDUP(s3);
		else
		    obj->full_path = STRDUP(s2);
	    }
	    else
	    {
		obj->full_path = STRDUP(url);
	    }
	    if(obj->full_path != NULL)
	    {
		const gchar *name = g_basename(obj->full_path);
		obj->name = STRDUP(
		    (name!= NULL) ? name : obj->full_path
		);
	    }

	    g_free(url);


	    buf += len;

	    buf++;			/* Skip deliminator */
	}

	return(glist);
}

/*
 *	Appends the Object to the DDE buffer.
 */
guint8 *ObjDDEBufferAppend(
	guint8 *buf, gint *buf_len,
	const obj_struct *obj
)
{
	gint i, len;
	gchar *url;

	if((obj == NULL) || (buf_len == NULL))
	    return(buf);

	if(obj->full_path == NULL)
	    return(buf);

	url = g_strconcat("file://", obj->full_path, NULL);

	/* Get current length and starting position */
	len = i = *buf_len;

	/* Calculate new buffer length needed */
	len += STRLEN(url);
	len++;			/* Null terminating byte / deliminator */

	/* Increase buffer allocation */
	buf = (guint8 *)g_realloc(
	    buf,
	    len * sizeof(guint8)
	);
	if(buf == NULL)
	{
	    *buf_len = 0;
	    return(buf);
	}

	/* Begin copying data to the newly allocated buffer segment */
	if(url != NULL)
	{
	    const gint url_len = STRLEN(url);
	    memcpy(buf + i, url, url_len);
	    i += url_len;
	}
	g_free(url);

	/* Null terminating byte */
	buf[i] = '\0';

	*buf_len = len;
	return(buf);
}
